package org.apache.bcel.util;

import java.io.Closeable;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.LinkOption;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Objects;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class ClassPath implements Closeable {
   private static final FilenameFilter ARCHIVE_FILTER = (dir, name) -> {
      name = name.toLowerCase(Locale.ENGLISH);
      return name.endsWith(".zip") || name.endsWith(".jar");
   };
   private static final FilenameFilter MODULES_FILTER = (dir, name) -> {
      name = name.toLowerCase(Locale.ENGLISH);
      return name.endsWith(".jmod");
   };
   public static final ClassPath SYSTEM_CLASS_PATH = new ClassPath(getClassPath());
   private final String classPath;
   private ClassPath parent;
   private final AbstractPathEntry[] paths;

   private static void addJdkModules(String javaHome, List<String> list) {
      String modulesPath = System.getProperty("java.modules.path");
      if (modulesPath == null || modulesPath.trim().isEmpty()) {
         modulesPath = javaHome + File.separator + "jmods";
      }

      File modulesDir = new File(modulesPath);
      if (modulesDir.exists()) {
         String[] modules = modulesDir.list(MODULES_FILTER);
         if (modules != null) {
            String[] var5 = modules;
            int var6 = modules.length;

            for(int var7 = 0; var7 < var6; ++var7) {
               String module = var5[var7];
               list.add(modulesDir.getPath() + File.separatorChar + module);
            }
         }
      }

   }

   public static String getClassPath() {
      String classPathProp = System.getProperty("java.class.path");
      String bootClassPathProp = System.getProperty("sun.boot.class.path");
      String extDirs = System.getProperty("java.ext.dirs");
      String javaHome = System.getProperty("java.home");
      List<String> list = new ArrayList();
      Path modulesPath = Paths.get(javaHome).resolve("lib/modules");
      if (Files.exists(modulesPath, new LinkOption[0]) && Files.isRegularFile(modulesPath, new LinkOption[0])) {
         list.add(modulesPath.toAbsolutePath().toString());
      }

      addJdkModules(javaHome, list);
      getPathComponents(classPathProp, list);
      getPathComponents(bootClassPathProp, list);
      List<String> dirs = new ArrayList();
      getPathComponents(extDirs, dirs);
      Iterator var7 = dirs.iterator();

      while(true) {
         File extDir;
         String[] extensions;
         do {
            String separator;
            if (!var7.hasNext()) {
               StringBuilder buf = new StringBuilder();
               separator = "";
               Iterator var16 = list.iterator();

               while(var16.hasNext()) {
                  String path = (String)var16.next();
                  buf.append(separator);
                  separator = File.pathSeparator;
                  buf.append(path);
               }

               return buf.toString().intern();
            }

            separator = (String)var7.next();
            extDir = new File(separator);
            extensions = extDir.list(ARCHIVE_FILTER);
         } while(extensions == null);

         String[] var11 = extensions;
         int var12 = extensions.length;

         for(int var13 = 0; var13 < var12; ++var13) {
            String extension = var11[var13];
            list.add(extDir.getPath() + File.separatorChar + extension);
         }
      }
   }

   private static void getPathComponents(String path, List<String> list) {
      if (path != null) {
         StringTokenizer tokenizer = new StringTokenizer(path, File.pathSeparator);

         while(tokenizer.hasMoreTokens()) {
            String name = tokenizer.nextToken();
            File file = new File(name);
            if (file.exists()) {
               list.add(name);
            }
         }
      }

   }

   static String packageToFolder(String name) {
      return name.replace('.', '/');
   }

   /** @deprecated */
   @Deprecated
   public ClassPath() {
      this(getClassPath());
   }

   public ClassPath(ClassPath parent, String classPath) {
      this(classPath);
      this.parent = parent;
   }

   public ClassPath(String classPath) {
      this.classPath = classPath;
      List<AbstractPathEntry> list = new ArrayList();
      StringTokenizer tokenizer = new StringTokenizer(classPath, File.pathSeparator);

      while(true) {
         String path;
         do {
            if (!tokenizer.hasMoreTokens()) {
               this.paths = new AbstractPathEntry[list.size()];
               list.toArray(this.paths);
               return;
            }

            path = tokenizer.nextToken();
         } while(path.isEmpty());

         File file = new File(path);

         try {
            if (file.exists()) {
               if (file.isDirectory()) {
                  list.add(new Dir(path));
               } else if (path.endsWith(".jmod")) {
                  list.add(new Module(new ZipFile(file)));
               } else if (path.endsWith(ModularRuntimeImage.MODULES_PATH)) {
                  list.add(new JrtModules(ModularRuntimeImage.MODULES_PATH));
               } else {
                  list.add(new Jar(new ZipFile(file)));
               }
            }
         } catch (IOException var7) {
            IOException e = var7;
            if (path.endsWith(".zip") || path.endsWith(".jar")) {
               System.err.println("CLASSPATH component " + file + ": " + e);
            }
         }
      }
   }

   public void close() throws IOException {
      if (this.paths != null) {
         AbstractPathEntry[] var1 = this.paths;
         int var2 = var1.length;

         for(int var3 = 0; var3 < var2; ++var3) {
            AbstractPathEntry path = var1[var3];
            path.close();
         }
      }

   }

   public boolean equals(Object o) {
      if (o instanceof ClassPath) {
         ClassPath cp = (ClassPath)o;
         return this.classPath.equals(cp.toString());
      } else {
         return false;
      }
   }

   public byte[] getBytes(String name) throws IOException {
      return this.getBytes(name, ".class");
   }

   public byte[] getBytes(String name, String suffix) throws IOException {
      DataInputStream dis = null;

      byte[] var7;
      try {
         InputStream inputStream = this.getInputStream(name, suffix);
         Throwable var5 = null;

         try {
            if (inputStream == null) {
               throw new IOException("Couldn't find: " + name + suffix);
            }

            dis = new DataInputStream(inputStream);
            byte[] bytes = new byte[inputStream.available()];
            dis.readFully(bytes);
            var7 = bytes;
         } catch (Throwable var23) {
            var5 = var23;
            throw var23;
         } finally {
            if (inputStream != null) {
               if (var5 != null) {
                  try {
                     inputStream.close();
                  } catch (Throwable var22) {
                     var5.addSuppressed(var22);
                  }
               } else {
                  inputStream.close();
               }
            }

         }
      } finally {
         if (dis != null) {
            dis.close();
         }

      }

      return var7;
   }

   public ClassFile getClassFile(String name) throws IOException {
      return this.getClassFile(name, ".class");
   }

   public ClassFile getClassFile(String name, String suffix) throws IOException {
      ClassFile cf = null;
      if (this.parent != null) {
         cf = this.parent.getClassFileInternal(name, suffix);
      }

      if (cf == null) {
         cf = this.getClassFileInternal(name, suffix);
      }

      if (cf != null) {
         return cf;
      } else {
         throw new IOException("Couldn't find: " + name + suffix);
      }
   }

   private ClassFile getClassFileInternal(String name, String suffix) {
      AbstractPathEntry[] var3 = this.paths;
      int var4 = var3.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         AbstractPathEntry path = var3[var5];
         ClassFile cf = path.getClassFile(name, suffix);
         if (cf != null) {
            return cf;
         }
      }

      return null;
   }

   public InputStream getInputStream(String name) throws IOException {
      return this.getInputStream(packageToFolder(name), ".class");
   }

   public InputStream getInputStream(String name, String suffix) throws IOException {
      InputStream inputStream = null;

      try {
         inputStream = this.getClass().getClassLoader().getResourceAsStream(name + suffix);
      } catch (Exception var5) {
      }

      return inputStream != null ? inputStream : this.getClassFile(name, suffix).getInputStream();
   }

   public String getPath(String name) throws IOException {
      int index = name.lastIndexOf(46);
      String suffix = "";
      if (index > 0) {
         suffix = name.substring(index);
         name = name.substring(0, index);
      }

      return this.getPath(name, suffix);
   }

   public String getPath(String name, String suffix) throws IOException {
      return this.getClassFile(name, suffix).getPath();
   }

   public URL getResource(String name) {
      AbstractPathEntry[] var2 = this.paths;
      int var3 = var2.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         AbstractPathEntry path = var2[var4];
         URL url;
         if ((url = path.getResource(name)) != null) {
            return url;
         }
      }

      return null;
   }

   public InputStream getResourceAsStream(String name) {
      AbstractPathEntry[] var2 = this.paths;
      int var3 = var2.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         AbstractPathEntry path = var2[var4];
         InputStream is;
         if ((is = path.getResourceAsStream(name)) != null) {
            return is;
         }
      }

      return null;
   }

   public Enumeration<URL> getResources(String name) {
      Vector<URL> results = new Vector();
      AbstractPathEntry[] var3 = this.paths;
      int var4 = var3.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         AbstractPathEntry path = var3[var5];
         URL url;
         if ((url = path.getResource(name)) != null) {
            results.add(url);
         }
      }

      return results.elements();
   }

   public int hashCode() {
      return this.parent != null ? this.classPath.hashCode() + this.parent.hashCode() : this.classPath.hashCode();
   }

   public String toString() {
      return this.parent != null ? this.parent + File.pathSeparator + this.classPath : this.classPath;
   }

   private static class Module extends AbstractZip {
      Module(ZipFile zip) {
         super(zip);
      }

      protected String toEntryName(String name, String suffix) {
         return "classes/" + ClassPath.packageToFolder(name) + suffix;
      }
   }

   private static class JrtModules extends AbstractPathEntry {
      private final ModularRuntimeImage modularRuntimeImage = new ModularRuntimeImage();
      private final JrtModule[] modules;

      public JrtModules(String path) throws IOException {
         super(null);
         this.modules = (JrtModule[])this.modularRuntimeImage.list(path).stream().map(JrtModule::new).toArray((x$0) -> {
            return new JrtModule[x$0];
         });
      }

      public void close() throws IOException {
         if (this.modules != null) {
            JrtModule[] var1 = this.modules;
            int var2 = var1.length;

            for(int var3 = 0; var3 < var2; ++var3) {
               JrtModule module = var1[var3];
               module.close();
            }
         }

         if (this.modularRuntimeImage != null) {
            this.modularRuntimeImage.close();
         }

      }

      ClassFile getClassFile(String name, String suffix) {
         JrtModule[] var3 = this.modules;
         int var4 = var3.length;

         for(int var5 = 0; var5 < var4; ++var5) {
            JrtModule module = var3[var5];
            ClassFile classFile = module.getClassFile(name, suffix);
            if (classFile != null) {
               return classFile;
            }
         }

         return null;
      }

      URL getResource(String name) {
         JrtModule[] var2 = this.modules;
         int var3 = var2.length;

         for(int var4 = 0; var4 < var3; ++var4) {
            JrtModule module = var2[var4];
            URL url = module.getResource(name);
            if (url != null) {
               return url;
            }
         }

         return null;
      }

      InputStream getResourceAsStream(String name) {
         JrtModule[] var2 = this.modules;
         int var3 = var2.length;

         for(int var4 = 0; var4 < var3; ++var4) {
            JrtModule module = var2[var4];
            InputStream inputStream = module.getResourceAsStream(name);
            if (inputStream != null) {
               return inputStream;
            }
         }

         return null;
      }

      public String toString() {
         return Arrays.toString(this.modules);
      }
   }

   private static class JrtModule extends AbstractPathEntry {
      private final Path modulePath;

      public JrtModule(Path modulePath) {
         super(null);
         this.modulePath = (Path)Objects.requireNonNull(modulePath, "modulePath");
      }

      public void close() throws IOException {
      }

      ClassFile getClassFile(String name, String suffix) {
         final Path resolved = this.modulePath.resolve(ClassPath.packageToFolder(name) + suffix);
         return Files.exists(resolved, new LinkOption[0]) ? new ClassFile() {
            public String getBase() {
               return Objects.toString(resolved.getFileName(), (String)null);
            }

            public InputStream getInputStream() throws IOException {
               return Files.newInputStream(resolved);
            }

            public String getPath() {
               return resolved.toString();
            }

            public long getSize() {
               try {
                  return Files.size(resolved);
               } catch (IOException var2) {
                  return 0L;
               }
            }

            public long getTime() {
               try {
                  return Files.getLastModifiedTime(resolved).toMillis();
               } catch (IOException var2) {
                  return 0L;
               }
            }
         } : null;
      }

      URL getResource(String name) {
         Path resovled = this.modulePath.resolve(name);

         try {
            return Files.exists(resovled, new LinkOption[0]) ? new URL("jrt:" + this.modulePath + "/" + name) : null;
         } catch (MalformedURLException var4) {
            return null;
         }
      }

      InputStream getResourceAsStream(String name) {
         try {
            return Files.newInputStream(this.modulePath.resolve(name));
         } catch (IOException var3) {
            return null;
         }
      }

      public String toString() {
         return this.modulePath.toString();
      }
   }

   private static class Jar extends AbstractZip {
      Jar(ZipFile zip) {
         super(zip);
      }

      protected String toEntryName(String name, String suffix) {
         return ClassPath.packageToFolder(name) + suffix;
      }
   }

   private static class Dir extends AbstractPathEntry {
      private final String dir;

      Dir(String d) {
         super(null);
         this.dir = d;
      }

      public void close() throws IOException {
      }

      ClassFile getClassFile(String name, String suffix) {
         final File file = new File(this.dir + File.separatorChar + name.replace('.', File.separatorChar) + suffix);
         return file.exists() ? new ClassFile() {
            public String getBase() {
               return Dir.this.dir;
            }

            public InputStream getInputStream() throws IOException {
               return new FileInputStream(file);
            }

            public String getPath() {
               try {
                  return file.getCanonicalPath();
               } catch (IOException var2) {
                  return null;
               }
            }

            public long getSize() {
               return file.length();
            }

            public long getTime() {
               return file.lastModified();
            }
         } : null;
      }

      URL getResource(String name) {
         File file = this.toFile(name);

         try {
            return file.exists() ? file.toURI().toURL() : null;
         } catch (MalformedURLException var4) {
            return null;
         }
      }

      InputStream getResourceAsStream(String name) {
         File file = this.toFile(name);

         try {
            return file.exists() ? new FileInputStream(file) : null;
         } catch (IOException var4) {
            return null;
         }
      }

      private File toFile(String name) {
         return new File(this.dir + File.separatorChar + name.replace('/', File.separatorChar));
      }

      public String toString() {
         return this.dir;
      }
   }

   public interface ClassFile {
      String getBase();

      InputStream getInputStream() throws IOException;

      String getPath();

      long getSize();

      long getTime();
   }

   private abstract static class AbstractZip extends AbstractPathEntry {
      private final ZipFile zipFile;

      AbstractZip(ZipFile zipFile) {
         super(null);
         this.zipFile = (ZipFile)Objects.requireNonNull(zipFile, "zipFile");
      }

      public void close() throws IOException {
         if (this.zipFile != null) {
            this.zipFile.close();
         }

      }

      ClassFile getClassFile(String name, String suffix) {
         final ZipEntry entry = this.zipFile.getEntry(this.toEntryName(name, suffix));
         return entry == null ? null : new ClassFile() {
            public String getBase() {
               return AbstractZip.this.zipFile.getName();
            }

            public InputStream getInputStream() throws IOException {
               return AbstractZip.this.zipFile.getInputStream(entry);
            }

            public String getPath() {
               return entry.toString();
            }

            public long getSize() {
               return entry.getSize();
            }

            public long getTime() {
               return entry.getTime();
            }
         };
      }

      URL getResource(String name) {
         ZipEntry entry = this.zipFile.getEntry(name);

         try {
            return entry != null ? new URL("jar:file:" + this.zipFile.getName() + "!/" + name) : null;
         } catch (MalformedURLException var4) {
            return null;
         }
      }

      InputStream getResourceAsStream(String name) {
         ZipEntry entry = this.zipFile.getEntry(name);

         try {
            return entry != null ? this.zipFile.getInputStream(entry) : null;
         } catch (IOException var4) {
            return null;
         }
      }

      protected abstract String toEntryName(String var1, String var2);

      public String toString() {
         return this.zipFile.getName();
      }
   }

   private abstract static class AbstractPathEntry implements Closeable {
      private AbstractPathEntry() {
      }

      abstract ClassFile getClassFile(String var1, String var2);

      abstract URL getResource(String var1);

      abstract InputStream getResourceAsStream(String var1);

      // $FF: synthetic method
      AbstractPathEntry(Object x0) {
         this();
      }
   }
}
